unit COMObjShellPropSheetExt;
{$DEFINE Shell471}
{..$DEFINE Shell500}
{$DEFINE Debug}
(*
   Daniel U. Thibault
   <D.U.Thibault@Bigfoot.com>
   3 March 1999
    1999 Daniel U. Thibault.  All Rights Reserved.

   This is freeware.  No legally binding promises.  If you use the
   component or any part of its code, you must include an
   acknowledgement in the finished code or executable (in the About
   box and/or the Help file).

   TCOMObjShellPropSheetExt:
   Generic TCOMObjShellExtInit descendant that implements the
   IShellPropSheetExt interface.

   The peculiar nature of this shell extension mandates a somewhat
   different approach from the programmer.
   The programmer should override TCOMObjShellPropSheetExt.DoAddPage
   to generate his property sheet page on-the-fly; he MUST also,
   within DoAddPage, assign the Page object reference to a persistent
   TPropertySheetPage variable --unless its not a keeper.

   TPropertySheetPage will include a host of methods to manipulate
   its dialog box template in memory.

   Actually, it isn't generic yet, but its getting better.

   #########

   Property Sheet handlers are registered to the target file type.

   For example,
   HKEY_CLASSES_ROOT\.del = DelShellFile
   HKEY_CLASSES_ROOT\DelShellFile\shellex\PropertySheetHandlers\DUTShellPropertySheet = CLSID_DelPropSheet

   You can register multiple property sheet handlers for a file type.
   In this case, the order of the subkey names in the PropertySheetHandlers
key determines the order of the additional property sheets.  You can use a
maximum of 24 (the value of MAXPROPPAGES) pages in a property sheet.

   The ReplacePage method is called only by Control Panel applications.
   It allows you to replace the property sheet of a standard Control Panel
application with a custom page.  For example, if a mouse manufacturer adds
extra buttons to its mouse, the manufacturer can replace the Buttons page
in the Control Panel's Mouse Properties property sheet.  The ReplacePage
method is not called by the shell because the shell does not have any
property pages that can be replaced by a shell extension.  Currently, only
Control Panel applications call this method, but other property sheet
suppliers could use it to allow their property pages to be replaced.
   Each property sheet handler that allows a property page to be replaced
must specify the registry location where other handlers that replace pages
register themselves.  For standard Control Panel applications, this
location is defined by the REGSTR_PATH_CONTROLSFOLDER constant in RegStr.pas.
This key, under the HKEY_LOCAL_MACHINE root, is where all Control Panel
property page replacement handlers must register.
   For example, a property sheet handler that needs to replace a property
page in the Mouse Properties Control Panel would register a property sheet
extension handler in the following registry location:
   HKEY_LOCAL_MACHINE\REGSTR_PATH_CONTROLSFOLDER\Mouse\shellex\PropertySheetHandlers = "NewMousePage"
i.e.
   HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Controls Folder\Mouse\shellex\PropertySheetHandlers = "NewMousePage"
   HKEY_LOCAL_MACHINE\...\PropertySheetHandlers\NewMousePage = CLSID
   In addition, a property sheet handler that allows replaceable pages must
define identifiers for each page that can be replaced.  Standard Control
Panel applications define these locations in CplExt.pas.
   CplExt.pas defines the subkey for each Control Panel application that
contains a replaceable property page: Mouse for a Mouse Properties Control
Panel application and Keyboard for a Keyboard Properties Control Panel application.
   (Well, actually it does not; they're mentioned as comments...)
   For example, CPLPAGE_MOUSE_BUTTONS is the identifier for the Buttons page
in the Mouse Properties property sheet, and CPLPAGE_KEYBOARD_SPEED is the
identifier for the Speed page in the Keyboard Properties property sheet.

   Of course, property sheet handlers may also be registered to Control Panel
applications so as to add pages (without replacing any).  This is the case with
the Display and Joystick Control Panels, for example:
HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Controls Folder\Display\shellex\PropertySheetHandlers\PlusPack CPL Extension = {41E300E0-78B6-11ce-849B-444553540000}
HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Controls Folder\Display\shellex\PropertySheetHandlers\DeskHtmlExt = {3FC0B520-68A9-11D0-8D77-00C04FD70822}
HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Controls Folder\Joystick\shellex\PropertySheetHandlers\Default Extension = {92187326-72B4-11d0-A1AC-0000F8026977}

   In the normal sequence of events,
TCOMObjShellPropSheetExt.AddPages is called first, then
TCOMObjShellPropSheetExt is released (freed), then
PropSheetCallback PSPCB_CREATE (if specified), then
PropSheetDlgProc WM_SETFONT (!), then
PropSheetDlgProc WM_INITDIALOG, then
more PropSheetDlgProc, then
PropSheetCallback PSPCB_RELEASE (if specified)
*)
interface

uses
   Classes,            { TStringList }
   Graphics,           { TIcon }
   Windows,
   Messages,           { WM_USER, WM_INITDIALOG, etc. }
   CommCtrl,           { TFNAddPropSheetPage, etc. }
   ActiveX,            { IDataObject }
   ShlObj,             { IShellPropSheetExt, etc. }
// COMObj,             { TComObjectFactory, etc. }
   COMObjShellExtInit, { TCOMObjShellExtInit }
   DUTDlgTemplate;     { TDUTDlgTemplate }

const
   //IDs of property sheet buttons
   ID_OKClose = ID_OK;  IDOKClose = ID_OKClose; { ID_OK = IDOK = 1 }
   //Note that the caption is "&Apply"
   ID_Apply = 12321;    IDApply = ID_Apply;
// ID_Cancel = 2;       IDCancel = ID_Cancel;
// ID_Help = 9;         IDHelp = ID_Help;

{$IFDEF Shell500}
   PSPCB_ADDREF = 0; //Requires Shell 5.0
{$ENDIF}

   PSP_PREMATURE         = $00000400;
{$IFDEF Shell471}
   //New flags for Wizard97; require shell 4.71 and up
   PSP_HIDEHEADER        = $00000800;
   PSP_USEHEADERTITLE    = $00001000;
   PSP_USEHEADERSUBTITLE = $00002000;
{$ENDIF}

   //Additional PSH_ constants:
   PSH_WIZARDHASFINISH   = $00000400; { replaces PSH_MULTILINETABS }
   PSH_RTLREADING        = $00000800;
   PSH_WIZARDCONTEXTHELP = $00001000;
{$IFDEF Shell471}
   //These new flags for Wizard97 require Shell 4.71 and up:
   PSH_WIZARD97          = $00002000;
   //Unused                $00004000
   PSH_WATERMARK         = $00008000;
   PSH_USEHBMWATERMARK   = $00010000;  // user pass in a hbmWatermark instead of pszbmWatermark
   PSH_USEHPLWATERMARK   = $00020000;  //
   PSH_STRETCHWATERMARK  = $00040000;  // stretchwatermark also applies for the header
   PSH_HEADER            = $00080000;
   PSH_USEHBMHEADER      = $00100000;
   PSH_USEPAGELANG       = $00200000;  // use frame dialog template matched to page
{$ENDIF}
{$IFDEF Shell500}
   //This new flag for Wizard-Lite requires Shell 5.0:
   PSH_WIZARD_LITE       = $00400000;
   PSH_NOCONTEXTHELP     = $00800000; //I'm guessing at the value
{$ENDIF}
   //The three wizard styles (PSH_WIZARD, PSH_WIZARD97 and PSH_WIZARD_LITE)
   //are mutually incompatible.

   PSCB_PRECREATE = 2;

{$IFDEF Shell471}
   //This requires Shell 4.71 and up:
   PSN_GETOBJECT            = PSN_FIRST - 10;
{$ENDIF}
{$IFDEF Shell500}
   //These require Shell 5.0:
   PSN_TRANSLATEACCELERATOR = PSN_FIRST - 12;
   PSN_QUERYINITIALFOCUS    = PSN_FIRST - 13;

   PSNRET_MESSAGEHANDLED = 3;
{$ENDIF}

   PSM_GETCURRENTPAGEHWND = WM_USER + 118;
// PropSheet_GetCurrentPageHwnd(hDlg) = (HWND)SndMsg(hDlg, PSM_GETCURRENTPAGEHWND, 0, 0L)
{$IFDEF Shell500}
   //This requires Shell 5.0:
   PSM_INSERTPAGE         = WM_USER + 119;
// PropSheet_InsertPage(hDlg, index, hpage) = SndMsg(hDlg, PSM_INSERTPAGE, (WPARAM)(index), (LPARAM)(hpage))
{$ENDIF}

   //Web Publishing Wizard OLE error messages:
   //Web Publishing Wizard first error message.
   WPWIZ_ERROR_FIRST = $40042000;
   //An unknown error occurred in the Web Publishing Wizard.
   WPWIZ_ERROR_UNKNOWN              = $C0042001;
   //The Web Publishing Wizard was unable to determine the version number of your service provider.
   WPWIZ_ERROR_PROV_QI              = $C0042002;
   //An error occurred while attempting to start the Web Publishing Wizard.
   WPWIZ_ERROR_INIT_FAILED          = $C0042003;
   //The Web Publishing Wizard is not installed correctly.  Please uninstall and then reinstall the wizard.
   WPWIZ_ERROR_COCREATE_WEBPOST     = $C0042004;
   //The Web Publishing Wizard could not run because there are no service providers installed.
   WPWIZ_ERROR_NO_PROVIDERS         = $C0042005;
   //An internal error (0x2006) occurred in the Web Publishing Wizard.
   WPWIZ_ERROR_STATE_PTR            = $C0042006;
   //An internal error (0x2007) occurred in the Web Publishing Wizard.
   WPWIZ_ERROR_WEBPOST_PTR          = $C0042007;
   //The file(s) you selected to publish could not be found.
   WPWIZ_ERROR_FILE_NOT_FOUND       = $C0042008;
   //An error occurred in the user interface of the Web Publishing Wizard.
   WPWIZ_ERROR_PROPSHEET_ERROR      = $C0042009;
   //The Web Publishing Wizard has run out of memory.  Please close other running applications before continuing.
   WPWIZ_ERROR_OUTOFMEMORY          = $C004200A;
   //Web Publishing Wizard last error message.
   WPWIZ_ERROR_LAST                 = $400420FF;

type
(*
   TPropSheetPage is augmented with shell 5.0
   These two pointers are appended:
   pszHeaderTitle    : PAnsiChar; //Addresses of strings displayed in the header
   pszHeaderSubTitle : PAnsiChar; //These are PWideChar in the Unicode (W) version

   TPropSheetHeader is augmented with shell 5.0
   These fields are appended:
   either
   hbmWatermark   : HBITMAP
   or
   pszbmWatermark : PAnsiChar { or PWideChar }
   then
   hplWatermark   : HPALETTE
   then either
   hbmHeader      : HBITMAP //Header bitmap shares the palette with watermark
   or
   pszbmHeader    : PAnsiChar { or PWideChar }

   The CommCtrl.pas definitions thus become:
*)
{$IFDEF Shell471}
   PPropSheetPageA = ^TPropSheetPageA;
   PPropSheetPageW = ^TPropSheetPageW;
   PPropSheetPage = PPropSheetPageA;

   TPropSheetPageA = record
      dwSize      : Longint;
      dwFlags     : Longint;
      hInstance   : THandle;
      case Integer of
      0: (
	   pszTemplate : PAnsiChar);
      1: (
	   pResource   : Pointer;
	   case Integer of
	   0: (
      hIcon       : THandle);
	   1: (
	   pszIcon     : PAnsiChar;
	   pszTitle    : PAnsiChar;
	   pfnDlgProc  : Pointer;
	   lParam      : Longint;
	   pfnCallback : TFNPSPCallbackA;
	   pcRefParent : PInteger;
      pszHeaderTitle    : PAnsiChar;
      pszHeaderSubTitle : PAnsiChar));
   end; { TPropSheetPageA }

   TPropSheetPageW = record
      dwSize      : Longint;
      dwFlags     : Longint;
      hInstance   : THandle;
      case Integer of
      0: (
      pszTemplate : PWideChar);
      1: (
      pResource   : Pointer;
      case Integer of
      0: (
	   hIcon       : THandle);
	   1: (
	   pszIcon     : PWideChar;
	   pszTitle    : PWideChar;
	   pfnDlgProc  : Pointer;
	   lParam      : Longint;
	   pfnCallback : TFNPSPCallbackW;
	   pcRefParent : PInteger;
      pszHeaderTitle    : PWideChar;
      pszHeaderSubTitle : PWideChar));
   end; { TPropSheetPageW }

   TPropSheetPage = TPropSheetPageA;

   PPropSheetHeaderA = ^TPropSheetHeaderA;
   PPropSheetHeaderW = ^TPropSheetHeaderW;
   PPropSheetHeader = PPropSheetHeaderA;

   TPropSheetHeaderA = record
      dwSize         : Longint;
      dwFlags        : Longint;
      hwndParent     : HWnd;
      hInstance      : THandle;
      case Integer of
      0: (
      hIcon          : THandle);
      1: (
      pszIcon        : PAnsiChar;
      pszCaption     : PAnsiChar;
      nPages         : Integer;
      case Integer of
      0: (
      nStartPage     : Integer);
      1: (
      pStartPage     : PAnsiChar;
      case Integer of
      0: (
      ppsp           : PPropSheetPageA);
      1: (
      phpage: Pointer;
      pfnCallback    : TFNPropSheetCallback;
      case Integer of
      0 : (
      hbmWatermark   : HBITMAP);
      1 : (
      pszbmWatermark : PAnsiChar;
      hplWatermark   : HPALETTE;
      case Integer of
      0 : (
      //Header bitmap shares the palette with watermark
      hbmHeader      : HBITMAP);
      1 : (
      pszbmHeader    : PAnsiChar)))));
   end; { TPropSheetHeaderA }

   TPropSheetHeaderW = record
      dwSize         : Longint;
      dwFlags        : Longint;
      hwndParent     : HWnd;
      hInstance      : THandle;
      case Integer of
      0: (
      hIcon          : THandle);
      1: (
      pszIcon        : PWideChar;
      pszCaption     : PWideChar;
      nPages         : Integer;
      case Integer of
      0: (
      nStartPage     : Integer);
      1: (
      pStartPage     : PWideChar;
      case Integer of
      0: (
      ppsp           : PPropSheetPageW);
      1: (
      phpage: Pointer;
      pfnCallback    : TFNPropSheetCallback;
      case Integer of
      0 : (
      hbmWatermark   : HBITMAP);
      1 : (
      pszbmWatermark : PWideChar;
      hplWatermark   : HPALETTE;
      case Integer of
      0 : (
      //Header bitmap shares the palette with watermark
      hbmHeader      : HBITMAP);
      1 : (
      pszbmHeader    : PWideChar)))));
   end; { TPropSheetHeaderW }

   TPropSheetHeader = TPropSheetHeaderA;

   //This requires Shell 4.71 and up:
   //PSN_GetObject's lParam
   PNMObjectNotify = ^TNMObjectNotify;
   TNMObjectNotify = packed record
      hdr     : TNMHdr;   //NMHDR
      iItem   : Integer;  //int
      piid    : TIID;     //const IID *
      pObject : Pointer;  //void *
      hResult : HRESULT;  //HRESULT
      //Control-specific flags (hints as to where in iItem it hit)
      dwFlags : DWord;    //DWORD
   end; { TNMObjectNotify }

   //Because of Delphi's strong typing, we must redeclare the externals:

function CreatePropertySheetPageA(var PSP: TPropSheetPageA): HPropSheetPage; stdcall;
function CreatePropertySheetPageW(var PSP: TPropSheetPageW): HPropSheetPage; stdcall;
function CreatePropertySheetPage(var PSP: TPropSheetPage): HPropSheetPage; stdcall;
function PropertySheetA(var PSH: TPropSheetHeaderA): Integer; stdcall;
function PropertySheetW(var PSH: TPropSheetHeaderW): Integer; stdcall;
function PropertySheet(var PSH: TPropSheetHeader): Integer; stdcall;

type
{$ENDIF}

   //PSN_Apply & PSN_Reset's lParam
   PPSHNotify = ^TPSHNotify;
   TPSHNotify = packed record
      Hdr    : TNMHdr;
      lParam : LPARAM;
   end; { TPSHNotify }

   TPropertySheetPage = class;

   TCOMObjShellPropSheetExt = class(TCOMObjShellExtInit, IShellPropSheetExt)
   protected
      function DoAddPage(Page : TPropertySheetPage) : Boolean; virtual;
      {
      This method is repeatedly called by IShellPropSheetExt.AddPages.
      In it, you fill in the property sheet page description.
      Return True if you wish to add the page to the property sheet;
      AddPages will call DoAddPage until it returns False.
      The default implementation creates an empty property sheet page.
      }
   public
      //IShellPropSheetExt
      function AddPages(lpfnAddPage : TFNAddPropSheetPage; lParam : LPARAM) : HResult; stdcall;
      function ReplacePage(uPageID : UINT; lpfnReplaceWith : TFNAddPropSheetPage; lParam : LPARAM) : HResult; stdcall;
   end;

   //TPropertySheetPage will include a host of methods to manipulate
   //its dialog box template in memory.
   //How the added children will expose their properties and event handlers
   //isn't settled yet...
   TPropertySheetPage = class(TObject)
   private
      FPageNumber : Integer;
      FPIDL       : PItemIDList;
      FDataObj    : IDataObject;
      FhKeyProgID : HKEY;
      FFiles      : TStringList;
      FIDPath     : String;

      FIcon       : TIcon;
      FTitle      : String;
      FTemplate   : TDUTDlgTemplate;
   protected
      hWndDlg  : HWND; //The property sheet page window handle (once known)
      TPSP     : TPropSheetPage;
      HPSP     : HPropSheetPage;
   public
      constructor Create(PageNumber : Integer; PIDL : PItemIDList; DataObj : IDataObject; hKeyProgID : HKEY; Files : TStringList; IDPath : String); virtual;
      destructor Destroy; override;
      function DoCreate : Integer; virtual;
      {
      This method is called by PropSheetCallback when processing PSPCB_CREATE.
      The default implementation returns non-zero to allow page creation.
      }
      procedure DoDestroy; virtual;
      {
      This method is called by PropSheetCallback when processing PSPCB_RELEASE.
      The default implementation does nothing.
      }

      property PageNumber : Integer
         read FPageNumber;
      {
      The property sheet page's number, starting at zero.
      }
      property PIDL       : PItemIDList
         read FPIDL;
      {
      Pointer to an ITEMIDLIST (item identifier list) that uniquely identifies the source folder.
      }
      property DataObj    : IDataObject
         read FDataObj;
      {
      Pointer to an IDataObject interface object that can be used to retrieve the objects being acted upon.
      }
      property hKeyProgID : HKEY
         read FhKeyProgID;
      {
      Registry key specifying the file class of the file object that has the focus.
      }
      property Files      : TStringList
         read FFiles;
      {
      The list of files being acted upon, when IDataObject can be rendered so, empty otherwise.
      }
      property IDPath     : String
         read FIDPath;
      {
      The path equivalent to FPIDL, when it exists.
      }

      property Icon : TIcon
         read FIcon
         write FIcon;
      {
      Icon to display on the tab of the page.
      }
      property Title : String
         read FTitle
         write FTitle;
      {
      The page's Title, which is displayed in the tab.
      }
      property Template : TDUTDlgTemplate
         read FTemplate;
      {
      The dialog box template.
      Methods will be provided for populating the template with buttons and stuff.
      }
   end; { TPropertySheetPage }

var
   DLLRefCount : Integer = 0;

implementation

uses
{$IFDEF Debug}
DLLDebug, { Loggit }
{$ENDIF}
   SysUtils,       { StrDispose, Format, etc. }
   COMServ,        { COMServer }
//This must be expelled to DUTShellPropertySheet
DUTDelShellUtils,
   DUTUtilsSys;       { GetChildByID }

{$IFDEF Shell471}
const
   cctrl = 'comctl32.dll';

function CreatePropertySheetPageA; external cctrl name 'CreatePropertySheetPageA';
function CreatePropertySheetPageW; external cctrl name 'CreatePropertySheetPageW';
function CreatePropertySheetPage; external cctrl name 'CreatePropertySheetPageA';
function PropertySheetA; external cctrl name 'PropertySheetA';
function PropertySheetW; external cctrl name 'PropertySheetW';
function PropertySheet; external cctrl name 'PropertySheetA';
{$ENDIF}

//These belong in the descendant DUTShellPropertySheet
{..$R DUTShellPropertySheet.RES}
{$I DUTShellPropertySheet.INC}

{ ########## TCOMObjShellPropSheetExt ########## }

function IsClose(hWndPS : HWnd) : Boolean;
{
   Returns True if the property sheet's OK button has become Close, False otherwise.
   Note: This function will probably need to be localized.
hWndPS
   Handle of the property sheet
   (You can typically use GetParent(hWndDlg), where hWndDlg is the page's handle)
}
var
   Buffer : Array[0..MAX_PATH] of Char; //Note that Buffer's length is MAX_PATH + 1
begin
   GetWindowText(GetChildByID(hWndPS, ID_OKClose), Buffer, MAX_PATH);
   //This may fail in other locales
   //Checking for "OK" rather than "Close" should be more robust, however.
   Result := (0 <> AnsiStrComp(Buffer, PChar('OK')));
// Result := (0 =  AnsiStrComp(Buffer, PChar('Close')));
end; { IsClose }

function PropSheetCallback(hWndDlg : HWnd; Msg : Integer; var TPSP : TPropSheetPage) : Integer; stdcall;
{
   This TFNPSPCallbackA function is an application-defined callback
   that a property sheet calls when a page is created and
   when it is about to be destroyed.
   An application can use this function to perform initialization
   and cleanup operations for the page.
   Note: PropSheetCallback CANNOT be made a method of TCOMObjShellPropSheetExt
   because the latter is unloaded right after AddPages concludes.
   ...But maybe it could be made a method of TPropertySheetPage?
   ...Probably not if PSPCB_RELEASE is to Free the TPropertySheetPage!

hWndDlg
   NULL (Reserved).

Msg
   Action flag.
   This parameter can be one of the following values:
   PSPCB_CREATE	A page is being created.
      Return nonzero to allow the page to be created or zero to prevent it.
   PSPCB_RELEASE	A page is being destroyed.
      The return value is ignored.
      Note that PSPCB_RELEASE can occur without a corresponding PSPCB_CREATE
      (if the user dismisses the property sheet without tabbing to our page).
   PSPCB_ADDREF (0) Requires Shell 5.0
      A page is being created. The return value is not used.

TPSP
   A TPropSheetPage reference that defines
   the page being created or destroyed.

   An application must specify the address of this callback
   in the pfnCallback member of a TPropSheetPage structure
   before specifying the address of the structure in a call
   to the CreatePropertySheetPage function.
}
begin
   Result := not 0; //Allow creation
   //A pointer to the TPropertySheetPage was appended to the TPropSheetPage
   //The TPropSheetPagePlus record allows access to both members
   case Msg of
{$IFDEF Shell500}
   PSPCB_ADDREF,
{$ENDIF}
   PSPCB_CREATE : begin
{$IFDEF Debug}
Loggit('PSPCB_ADDREF/PSPCB_CREATE');
{$ENDIF}
      Result := TPropertySheetPage(TPSP.lParam).DoCreate;
{$IFDEF Debug}
Loggit('PSPCB_ADDREF/PSPCB_CREATE: DoCreate done');
{$ENDIF}
      //Result non-zero means its OK to create the page
      //PSPCB_ADDREF ignores the return value.
      end; { PSPCB_ADDREF, PSPCB_CREATE }
   PSPCB_RELEASE : begin
{$IFDEF Debug}
Loggit('PSPCB_RELEASE');
{$ENDIF}
      TPropertySheetPage(TPSP.lParam).DoDestroy;
{$IFDEF Debug}
Loggit('PSPCB_RELEASE: DoDestroy done');
{$ENDIF}
      //Result is ignored
      //The shell will free the TPropSheetPage;
      //we should, however, free the TPropertySheetPage
      TPropertySheetPage(TPSP.lParam).Free;
{$IFDEF Debug}
Loggit('PSPCB_RELEASE: TPropertySheetPage Freed');
{$ENDIF}
      end; { PSPCB_RELEASE }
   end; { case }
end; { PropSheetCallback }

function PropSheetDlgProc(hWndDlg : HWnd; Msg : UINT; wParam : wParam; lParam : LPARAM) : Bool; stdcall;
{
   The dialog box procedure for the page.
   The procedure must not call the EndDialog function.
   Returns True if it processes the message, False if it does not (except in the WM_INITDIALOG case; see below).
   Note: PropSheetDlgProc CANNOT be made a method of TCOMObjShellPropSheetExt.
         (Although it is called, the arguments seem to take on random values)
   ...But maybe it could be made a method of TPropertySheetPage?

hWndDlg
   The window handle of the property sheet page.

   Notes:
   It is possible to add pages to and remove pages from the property sheet
   programmatically, with the PSM_ADDPAGE and PSM_REMOVEPAGE messages.
   You can select a page with the PSM_SETCURSEL or PSM_SETCURSELID message.
   You can set the property sheet's title with the PSM_SETTITLE message.
   You can "click" the Apply button with the PSM_APPLY message.
   I'm not sure about PSM_PRESSBUTTON (haven't tried it), but I guess
   it could be used to click other buttons (OK/Close, Cancel, Help).

   To provide context-sensitive help in a dialog box, you must create an array
   consisting of pairs of doubleword values.
   This array is used in the WM_HELP and WM_CONTEXTMENU message handlers.
   The first value in each pair is the ID of a control in the dialog box,
   and the second is the context identifier of the help topic for the control.
   The array should contain one pair of identifiers for each control in the dialog box.
   The array is concluded with a double-null entry.
   The WinHelp function searches the array for the ID of the specified control
   and then retrieves the corresponding help context identifier.
   If the control has an ID of -1, the system searches for the next control
   that is a tab stop and uses its ID to find the help context identifier.
   For this reason, it is important that you add static text before controls in
   a template.
   Should there be no topic corresponding to the context ID, WinHelp displays a
   popup that simply states "No Help topic is associated with this item".
}
var
   PSP     : TPropertySheetPage;
   Index,
   Num     : Integer;
   Buffer  : Array[0..MAX_PATH] of Char;
   AllZero : Boolean;
   PBuffer : PChar;
begin
   Result := False; //Allow default message handling
   //PSP will be invalid until WM_INITDIALOG is processed
   PSP := TPropertySheetPage(GetWindowLong(hWndDlg, DWL_USER));
   case Msg of
   WM_SETFONT : begin
      //Note that this message typically precedes WM_INITDIALOG
      //wParam = (WPARAM) hfont;            // handle of font
      //lParam = MAKELPARAM(fRedraw, 0);    // redraw flag
{$IFDEF Debug}
Loggit('PropSheetDlgProc WM_SETFONT');
{$ENDIF}
   end; { WM_SETFONT }
   WM_INITDIALOG : begin
{$IFDEF Debug}
Loggit('PropSheetDlgProc WM_INITDIALOG');
{$ENDIF}
{
   wParam = hwndFocus
      Identifies the control to receive the default keyboard focus.
      Windows assigns the default keyboard focus only if the dialog box procedure returns TRUE.
   lParam = lInitParam
      Specifies additional initialization data.
      It is a pointer to a TPropSheetPage.
      This data is passed to Windows as the lParamInit or dwInitParam parameter
      in a call to the CreateDialogIndirectParam, CreateDialogParam,
      DialogBoxIndirectParam, or DialogBoxParam function used to create the dialog box.
      This parameter is zero if any other dialog box creation function is used.
}
      //Store the TPropertySheetPage for future reference
      PSP := TPropertySheetPage(PPropSheetPage(lParam)^.lParam);
      SetWindowLong(hWndDlg, DWL_USER, Integer(PSP));
      //Note the hWndDlg for future reference
      PSP.hWndDlg := hWndDlg;
      //Restore AcceptDroppedFiles if required
      if PSP.Template.AcceptDroppedFiles then SetWindowLong(hWndDlg, GWL_EXSTYLE, (WS_EX_ACCEPTFILES or GetWindowLong(hWndDlg, GWL_EXSTYLE)));

//The block beginning here needs to be customizable
      {
      Set the tab-stop positions in the list box (it must have been created with the LBS_USETABSTOPS style).
      cTabs (wParam). Specifies the number of tab stops in the list box.
         If 1, sets tab stops at lpnTabs intervals.
      lpnTabs (lParam). Points to the first member of an array of integers containing the tab stops, in dialog box units.
         The tab stops must be sorted in ascending order; backward tabs are not allowed.
      }
      Num := 20; //Can't pass a pointer to a constant
      //hWndDlg, the page, owns the controls that are on it.
      SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_SETTABSTOPS, 1, Integer(@Num));
      with TPropertySheetPage(PPropSheetPage(lParam)^.lParam).FFiles do begin
         { Set the static text }
         if Count > 1 then begin
            SetDlgItemText(hWndDlg, IDC_Static, PChar(Format('%d files selected', [Count])));
         end else begin
            SetDlgItemText(hWndDlg, IDC_Static, PChar(Format('%d file selected', [Count])));
         end; { else }
         { Fill the list box }
         AllZero := True;
         for Index := 0 to (Count - 1) do begin
            Num := GetDelFileNumber(Strings[Index]);
            if Num < 0 then begin
               SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_ADDSTRING, 0, Integer(PChar('bad'#9 + Strings[Index])));
               AllZero := False;
            end else begin
               SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_ADDSTRING, 0, Integer(PChar(Format('%d'#9'%s', [Num, Strings[Index]]))));
               AllZero := AllZero and (Num = 0);
            end; { else }
         end; { for }
      end; { with }
      if AllZero then begin
         //Disable Zero-Out button
         EnableWindow(GetDlgItem(hWndDlg, IDC_Button), False);
      end; { then }
//The block ending here needs to be customizable
      { Set the message return value }
      SetWindowLong(hWndDlg, DWL_MSGRESULT, 0);
      {
         In the special case of WM_INITDIALOG, a True Result
         means SetFocus was not called and the system should
         give the focus to the first suitable control
      }
//This Result needs to be customizable
      Result := True;
      end; { WM_INITDIALOG }

   WM_NOTIFY : begin
{
   wParam = IDCtrl   ID of control sending the message
   lParam = pNMH     Pointer to a TNMHdr structure;
                     the hWndFrom member is the property sheet handle.
                     Note that GetParent(hWndDlg) = hWndFrom
                     That is to say,
                     the property sheet owns
                     the property sheet page which owns
                     its child controls.
}
      case PNMHdr(lParam)^.Code of
      PSN_SETACTIVE : begin
{$IFDEF Debug}
Loggit('PSN_SETACTIVE');
{$ENDIF}
         { The property sheet page is receiving the focus }
//This needs to be a customizable event
         {
            Return True (via DWL_MSGRESULT) to prevent the page from
            getting the activation or False to allow it.
            To set the activation to a particular page,
            return the resource identifier of the page.
            An application can use this notification to initialize data in the page.
         }
         { Set the message return value }
         SetWindowLong(hWndDlg, DWL_MSGRESULT, Integer(False));
         Result := True; //Required for the PSNRET_ value to take effect
         end; { PSN_SETACTIVE }

      PSN_KILLACTIVE : begin
{$IFDEF Debug}
Loggit('PSN_KILLACTIVE');
{$ENDIF}
         { The property sheet page is losing the focus }
//This needs to be a customizable event
         {
            This is either because another page is being activated
            or the user has chosen the OK button.
            Return True (via DWL_MSGRESULT) to prevent the page from
            losing the activation or False to allow it.
            In the latter case a PSN_APPLY will follow if the user had clicked OK.
         }
         { Set the message return value }
         SetWindowLong(hWndDlg, DWL_MSGRESULT, LongInt(False));
         Result := True; //Required for the PSNRET_ value to take effect
         end; { PSN_KILLACTIVE }

      PSN_HELP : begin
{$IFDEF Debug}
Loggit('PSN_HELP');
{$ENDIF}
         { The user clicked the Help button }
//This needs to be a customizable event
         //Invoke Help here
         if PSP.Template.HelpContext = 0 then begin
            //Display the Help topics window
            //(typically when no context-specific Help is available)
            WinHelp(hWndDlg, PChar(PSP.Template.HelpFile), HELP_FINDER, PSP.Template.HelpContext);
         end else if PSP.Template.HelpContext > 0 then begin
            //Display a specific topic
            WinHelp(hWndDlg, PChar(PSP.Template.HelpFile), HELP_CONTEXT, PSP.Template.HelpContext);
         end else begin
            //To display a specific topic in a popup window
            WinHelp(hWndDlg, PChar(PSP.Template.HelpFile), HELP_CONTEXTPOPUP, -PSP.Template.HelpContext);
         end; { else }
         { No return value }
         Result := False;
         //Note: If PSN_HELP is invoked, we must send a HELP_QUIT upon closing the page.
         //(Because the window handle passed in the WinHelp call must match,
         // we cannot do this in the PSPCB_RELEASE event; instead we do it
         // in the PSN_APPLY and PSN_RESET events)
         end; { PSN_HELP }

      PSN_APPLY : begin
{$IFDEF Debug}
Loggit('PSN_APPLY');
{$ENDIF}
         { The user chose one of the OK/Close or Apply buttons, even from a different page }
//This needs to be a customizable event
         { Return values:
            PSNRET_INVALID_NOCHANGEPAGE Prevents the changes from taking effect and
               returns the focus to the page.
            PSNRET_NOERROR              Accepts the changes and
               allows the property sheet to be destroyed.
         {
            To receive this notification, a page must allow the PSN_KILLACTIVE
            notification message to go through (see above).
            (The Win32.hlp documentation is incorrect)
            PPSHNotify(lParam)^.lParam is TRUE (1) if the user clicked the OK/Close button,
            and is FALSE (0) if the user clicked the Apply button.
            Use the IsClose(hWndDlg) function to find out if OK has become Close.
         }
         //Obtain the number of items in the list box
         Num := SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_GETCOUNT, 0, 0);
         {
         We'll use AllZero as a flag that there were changes to the files.
         If AllZero is still True after checking each file, there were no changes.
         We must do this to distinguish between OK and Close.
         }
         AllZero := True;
         for Index := 0 to (Num - 1) do begin
            //Obtain Text[Index]
            SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_GETTEXT, Index, LongInt(@Buffer));
            //Is the magic number in the list box zero?
            if Buffer[0] = '0' then begin
               //Skip ahead of the imbedded tab
               PBuffer := StrScan(Buffer, #9) + 1;
               //Is the magic number in the file non-zero?
               if GetDelFileNumber(String(PBuffer)) > 0 then begin
                  SetDelFileNumber(String(PBuffer), 0);
                  AllZero := False;
                  //Notify the shell of the change
                  SHChangeNotify(SHCNE_UPDATEITEM, SHCNF_PATH or SHCNF_FLUSHNOWAIT, PBuffer, Nil);
               end; { then }
            end; { then }
         end; { for }
         //Each page must mark itself as "unchanged" before the Apply button is disabled.
         { Instead of PNMHdr(lParam)^.hWndFrom, we could use GetParent(hWndDlg) }
         SendMessage(PNMHdr(lParam)^.hWndFrom, PSM_UNCHANGED, hWndDlg, 0);
         if not AllZero then begin
            //The changes are now irreversible
            //(This disables the Cancel button and changes the OK button into a Close button;
            // note that SendMessage will NOT work properly)
            PostMessage(PNMHdr(lParam)^.hWndFrom, PSM_CANCELTOCLOSE, 0, 0);
         end; { then }
         //If the changes require a Windows restart, must send a PSM_RESTARTWINDOWS
         //If the changes require a reboot, must send a PSM_REBOOTSYSTEM
         if PPSHNotify(lParam)^.lParam = 1 then begin
            { OK/Close was clicked: Close Help if previously invoked }
            WinHelp(hWndDlg, PChar(PSP.Template.HelpFile), HELP_QUIT, 0);
         end; { then }
         { Set the message return value }
         SetWindowLong(hWndDlg, DWL_MSGRESULT, PSNRET_NOERROR);
         Result := True; //Required for the PSNRET_ value to take effect
         end; { PSN_APPLY }

      PSN_QUERYCANCEL : begin
{$IFDEF Debug}
Loggit('PSN_QUERYCANCEL');
{$ENDIF}
//This needs to be a customizable event
         { The user clicked the Cancel button }
         {
            Return TRUE (via DWL_MSGRESULT) to prevent the cancel
            or FALSE to allow it.  In the latter case a PSN_RESET will follow.
         }
         { Set the message return value }
         SetWindowLong(hWndDlg, DWL_MSGRESULT, LongInt(False));
         Result := True; //Required for the PSNRET_ value to take effect
         end; { PSN_QUERYCANCEL }

      PSN_RESET : begin
{$IFDEF Debug}
Loggit('PSN_RESET');
{$ENDIF}
//This needs to be a customizable event
         { The user clicked the Cancel button (or the close box) and QueryCancel went through }
         {
            All changes made since the user last chose the Apply button are canceled.
            To receive this notification, a page must set the DWL_MSGRESULT value
            to FALSE in response the PSN_QUERYCANCEL notification message (see above).
            (The Win32.hlp documentation is incorrect)
            PPSHNotify(lParam)^.lParam is TRUE (1) if the user clicked the sheet's close box,
            and is FALSE (0) if the user clicked the Cancel button.
         }
         //Normally we should reset the list box and then send a PSM_UNCHANGED
         //but in this case the property page is about to be destroyed anyway...
         { Close Help if previously invoked }
         WinHelp(hWndDlg, PChar(PSP.Template.HelpFile), HELP_QUIT, 0);
         { No return value }
         Result := False;
         end; { PSN_RESET }
{$IFDEF Shell471}
      PSN_GETOBJECT : begin
{$IFDEF Debug}
Loggit('PSN_GETOBJECT');
{$ENDIF}
         {
         Sent by a property sheet to request an OLE drop target object
         when the cursor passes over one of the tabs
         }
         PNMObjectNotify(lParam)^.pObject := Nil;
         PNMObjectNotify(lParam)^.hResult := E_NOTIMPL;
         { Set the message return value }
         SetWindowLong(hWndDlg, DWL_MSGRESULT, LongInt(False));
         Result := True;
         end; { PSN_GETOBJECT }
{$ENDIF}
{$IFDEF Shell500}
      PSN_TRANSLATEACCELERATOR : begin
         {
         lParam may be typecast as a PPSHNotify
         TSHNotify.Hdr.hWndFrom contains the window handle to the property sheet;
         TSHNotify.lParam is a pointer to the message's TMsg structure
         --typecast it to a PMsg to get access to the parameters of the message to be translated.
         }
         with PMsg(PPSHNotify(lParam)^.lParam)^ do begin
//          ...
         end; { with }
         { Set the message return value }
         SetWindowLong(hWndDlg, DWL_MSGRESULT, PSNRET_MESSAGEHANDLED);
         Result := True; //Required for the PSNRET_ value to take effect
         end; { PSN_TRANSLATEACCELERATOR }

      PSN_QUERYINITIALFOCUS : begin
         {
         lParam may be typecast as a PPSHNotify
         TSHNotify.Hdr.hWndFrom contains the window handle to the property sheet;
         TSHNotify.lParam is the window handle of the control that will be given the focus by default.
      	To specify which control should receive focus, return the control's window handle.
         Otherwise, return zero.
         }
         { Set the message return value }
         SetWindowLong(hWndDlg, DWL_MSGRESULT, 0);
         Result := True; //Required for the PSNRET_ value to take effect
         end; { PSN_QUERYINITIALFOCUS }
{$ENDIF}
      end; { case }
      end; { WM_NOTIFY }

   WM_COMMAND : begin
//This needs to be a customizable set of events
//Basically, we must route these events to a user "form" and back...
{$IFDEF Debug}
Loggit('PropSheetDlgProc WM_COMMAND');
{$ENDIF}
{
   HiWord(wParam) = wNotifyCode
   LoWord(wParam) = wID          Item, control, or accelerator identifier
   lParam         = hWndCtl      hWnd of control
                                 Typically, hWndCtl is owned by hWndDlg
}
      case LoWord(wParam) of
      IDC_Button : begin
         { The "zero out" button was clicked }
         //Obtain the number of items in the list box
         Num := SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_GETCOUNT, 0, 0);
         for Index := 0 to (Num - 1) do begin
            //Obtain Text[Index]
            SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_GETTEXT, Index, LongInt(@Buffer));
            //Delete it
            SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_DELETESTRING, Index, 0);
            //Skip ahead of the imbedded tab
            PBuffer := StrScan(Buffer, #9) + 1;
            //We don't set the magic number to zero yet; we'll do that in the PSN_APPLY event.
            //Re-insert the Text[Index]
            SendDlgItemMessage(hWndDlg, IDC_ListBox, LB_INSERTSTRING, Index, Integer(PChar('0'#9 + String(PBuffer))));
         end; { for }
         { Disable the button }
         EnableWindow(GetDlgItem(hWndDlg, IDC_Button), False);
         { Advise the sheet that Apply should be enabled }
         SendMessage(GetParent(hWndDlg), PSM_CHANGED, hWndDlg, 0);
         //Result is False, which means we've processed the message
         end; { IDC_Button }

      else
         { no action }
      end; { case }
      end; { WM_COMMAND }

   WM_HELP : begin
   { Property sheets have the dwStyle DS_CONTEXTHELP }
{
   To provide context-sensitive help in a dialog box, the dialog box procedure
   must process the WM_HELP and WM_CONTEXTMENU messages.
   The dialog box procedure receives WM_HELP when the user presses the F1 key or
   uses the title bar help box; it receives WM_CONTEXTMENU when the user clicks
   the right mouse button.

   The lParam parameter of WM_HELP is a pointer to a HELPINFO structure.
   The hItemHandle member of this structure will be the handle of the control
   for which the user has requested help.
   You must pass this handle to the WinHelp function along with the HELP_WM_HELP
   command, the name of your help file, and a pointer to the array of identifiers.
}
{$IFDEF Debug}
Loggit('WM_HELP: PHelpInfo(lParam)^.hItemHandle = ' + IntToStr(PHelpInfo(lParam)^.hItemHandle));
{$ENDIF}
//This needs to be a customizable event
   WinHelp(PHelpInfo(lParam)^.hItemHandle, PChar(PSP.Template.HelpFile), HELP_WM_HELP, Integer(PSP.Template.HelpContextArray));
   end; { WM_HELP }

   WM_CONTEXTMENU : begin
{
   Processing WM_CONTEXTMENU is similar to processing WM_HELP when calling the
   WinHelp function, with these two exceptions:

   1) You pass the wParam parameter from WM_CONTEXTMENU, which is the handle of
   the control that sent the message.
   2) You specify the HELP_CONTEXTMENU command instead of HELP_WM_HELP.

   The HELP_CONTEXTMENU command causes Windows Help to display a menu before it
   displays the help topic.  The menu is system defined.  It consists of the
   single entry "What's This?" (plus any relevant context commands such as
   Cut/Copy/Paste).
}
{$IFDEF Debug}
Loggit('WM_CONTEXTMENU: wParam = ' + IntToStr(wParam));
{$ENDIF}
//This needs to be a customizable event
   WinHelp(wParam, PChar(PSP.Template.HelpFile), HELP_CONTEXTMENU, Integer(PSP.Template.HelpContextArray));
   end; { WM_CONTEXTMENU }

   WM_DROPFILES : begin
{$IFDEF Debug}
Loggit('PropSheetDlgProc WM_DROPFILES');
{$ENDIF}
   end; { WM_DROPFILES }

   //Other messages identified so far:
WM_DESTROY,
WM_MOVE,
WM_SIZE,
WM_ACTIVATE,
WM_SETFOCUS,
WM_KILLFOCUS,
WM_GETTEXT, { on alt key stroke?? }
WM_PAINT,
WM_ERASEBKGND,
WM_SHOWWINDOW,
WM_ACTIVATEAPP,
WM_SETCURSOR,
WM_MOUSEACTIVATE,
WM_CHILDACTIVATE,
WM_WINDOWPOSCHANGING,
WM_WINDOWPOSCHANGED,
WM_STYLECHANGING, //When using SetWindowLong GWL_EXSTYLE or GWL_STYLE
WM_STYLECHANGED,  //When using SetWindowLong GWL_EXSTYLE or GWL_STYLE
WM_NCDESTROY,
WM_NCCALCSIZE,
WM_NCHITTEST,
WM_NCPAINT,
WM_GETDLGCODE,
WM_CTLCOLORDLG,
WM_KEYUP,
WM_SYSKEYDOWN,
WM_SYSKEYUP,
WM_CTLCOLORLISTBOX,
WM_CTLCOLORBTN,
WM_CTLCOLORSTATIC,
WM_MOUSEMOVE,
WM_LBUTTONDOWN,
WM_LBUTTONUP,
WM_LBUTTONDBLCLK,
WM_RBUTTONDOWN,
WM_RBUTTONUP,
WM_RBUTTONDBLCLK,
WM_MBUTTONDOWN,
WM_MBUTTONUP,
WM_MBUTTONDBLCLK,
WM_MOUSEWHEEL,
//DM_GETDEFID,   { = WM_USER + 0 } //Get the ID of the default push button control
DM_SETDEFID      { = WM_USER + 1 } //Set the ID of the default push button control
//DM_REPOSITION, { = WM_USER + 2 } //Reposition a top-level dialog box so that it fits within the desktop area
//PSM_PAGEINFO,  { = WM_USER + 100) //Property sheet message (undocumented)
//PSM_SHEETINFO  { = WM_USER + 101) //Property sheet message (undocumented)
   : begin
{
WM_USER + 1, to look only at that one, is used by several Windows controls:
            TB_ENABLEBUTTON (Toolbar)
            TBM_GETRANGEMIN (TrackBar)
            RB_INSERTBANDA (ReBar)
            TTM_ACTIVATE (ToolTips)
            PBM_SETRANGE (Progress)
            SB_SETTEXTA (StatusBar)
            HKM_SETHOTKEY (HotKey)
            WM_CHOOSEFONT_GETLOGFONT (Font dialog box)
            WM_PSD_FULLPAGERECT (Page Setup dialog box)
            DM_SETDEFID (change the identifier of the default push button for a dialog box)
}
      { no action }
      end; { WM_... }
   else
{$IFDEF Debug}
Loggit('PropSheetDlgProc else Msg = ' + IntToStr(Msg) + ' wParam = ' + IntToStr(wParam) + ' lParam = ' + IntToStr(lParam));
{$ENDIF}
      { No action }
   end; { case }
end; { PropSheetDlgProc }

{ #################### TPropertySheetPage #################### }
(*
  PDlgTemplate = ^TDlgTemplate;
  TDlgTemplate = packed record
    style: DWORD;
    dwExtendedStyle: DWORD;
    cdit: Word;
    x: SHORT;
    y: SHORT;
    cx: SHORT;
    cy: SHORT;
  end; { DLGTEMPLATE }

  { 32 bit Dialog item template. }
  PDlgItemTemplate = ^TDlgItemTemplate;
  TDlgItemTemplate = packed record
    style: DWORD;
    dwExtendedStyle: DWORD;
    x: SHORT;
    y: SHORT;
    cx: SHORT;
    cy: SHORT;
    id: Word;
  end; { DLGITEMTEMPLATE }
*)
constructor TPropertySheetPage.Create(PageNumber : Integer; PIDL : PItemIDList; DataObj : IDataObject; hKeyProgID : HKEY; Files : TStringList; IDPath : String);
begin
{$IFDEF Debug}
Loggit('TPropertySheetPage.Create begins');
{$ENDIF}
   inherited Create;
   FPageNumber := PageNumber;
   FPIDL       := PIDL;
   FDataObj    := DataObj;
   FhKeyProgID := hKeyProgID;
   FFiles      := Files;
   FIDPath     := IDPath;

   hWndDlg    := 0; //Unknown as yet
   FIcon     := TIcon.Create;
   FTitle    := IntToStr(PageNumber);
   FTemplate := TDUTDlgTemplate.Create;
{$IFDEF Debug}
Loggit('TPropertySheetPage.Create ends');
{$ENDIF}
end; { TPropertySheetPage.Create }

destructor TPropertySheetPage.Destroy;
begin
// FFiles.Free; //The ancestor COMObjShellExtInit will Free it
   FIcon.Free;
   FTemplate.Free;
   inherited;
end; { TPropertySheetPage.Destroy }

function TPropertySheetPage.DoCreate : Integer;
{
This method is called by PropSheetCallback when processing PSPCB_CREATE.
The default implementation returns non-zero to allow page creation.
}
begin
   Result := not 0;
end; { TPropertySheetPage.DoCreate }

procedure TPropertySheetPage.DoDestroy;
{
This method is called by PropSheetCallback when processing PSPCB_RELEASE.
TPropertySheetPage will be Freed immediately after this call concludes.
The default implementation does nothing.
}
begin
   { no action }
end; { TPropertySheetPage.DoDestroy }

{ #################### TCOMObjShellPropSheetExt #################### }

function TCOMObjShellPropSheetExt.DoAddPage(Page : TPropertySheetPage) : Boolean;
{
   DoAddPage is called repeatedly by AddPages.
   Page is partially filled in, with the following fields valid:
   PageNumber  Starting at zero, this numbers the property sheet pages as they are added.
   PIDL        \
   DataObj      \
   hKeyProgID    These other fields are as described in TCOMObjShellExtInit
   Files        /
   IDPath      /

   DoAddPage should return False if the Page should not be added.
   Thus, to add a single page you would return True on the first DoAddPage call
   (after filling in the Page) and False on the second one.

   This default implementation creates a single empty Page.
}
begin
   Result := (Page.PageNumber = 0);
end; { TCOMObjShellPropSheetExt.DoAddPage }

function TCOMObjShellPropSheetExt.AddPages(lpfnAddPage : TFNAddPropSheetPage; lParam : LPARAM) : HResult; stdcall;
{
   For each page it needs to add to a property sheet,
   a property sheet handler fills a PROPSHEETPAGE structure,
   calls CreatePropertySheetPage, and then
   calls the function specified by lpfnAddPage.
   AddPages is called when the property sheet first needs to be displayed
   --i.e., when the user clicks on the tab.
   IMPORTANT: TCOMObjShellPropSheetExt is Freed after AddPages completes!

lpfnAddPage (AddPropSheetPageProc)
   Pointer to a function that the property sheet handler must call
   to add a page to the property sheet.
   This function expects a property sheet handle (returned by a call
   to the CreatePropertySheetPage function) and
   the lParam parameter passed to the AddPages method.

lParam
   Parameter to pass to the function specified by lpfnAddPage.
   It seems to be a PPropSheetHeader.

HResult
   AddPages should return S_OK if successful.  If the method fails, an
OLE-defined error code should be returned.
   With version 4.71 and later of the Shell, you can request that a particular
property sheet page be displayed first, instead of the default page.  To do so,
return the one-based index of the desired page.  For example, if you want the
second of three pages displayed, the return value should be 2.  Note that this
return value is a request.  The property sheet may still display the default page.
   How do you know which number your page gets assigned to?  By consulting
   the PPropSheetHeader(lParam).  Its nPages member gives you the number of
   already-created pages.  Your number will be nPages + 1.
   Neither PSH_PROPSHEETPAGE nor PSH_USEPSTARTPAGE seem to be used by the shell,
   so nStartPage seems to be meaningful.
}
var
   PageNumber : Integer;
   Page       : TPropertySheetPage;
begin
{$IFDEF Debug}
Loggit('AddPages begins');
{$ENDIF}
   PageNumber := 1 + PPropSheetHeader(lParam).nPages;
   Result := E_UNEXPECTED;
   try
      repeat
         Page := TPropertySheetPage.Create(PageNumber, FPIDL, FDataObj, FhKeyProgID, FFiles, FIDPath);
      if not DoAddPage(Page) then break;
{$IFDEF Debug}
Loggit('AddPages: Adding the page');
{$ENDIF}
         //Size, in bytes, of the TPropSheetPage structure.
         //The size includes any extra application-defined data at the end of the structure.
         //The property sheet manager uses this member to determine which version of the PROPSHEETHEADER structure you are using.
         Page.TPSP.dwSize := SizeOf(Page.TPSP);

         (*
         Possible flags:
         PSP_DEFAULT      Uses the default meaning for all structure members.
         PSP_DLGINDIRECT  Uses pResource instead of pszTemplate. That is, the template for the property sheet dialog box is supplied in memory, not loaded from a resource.
         PSP_HASHELP      Enables the property sheet Help button when this page is active.
         PSP_PREMATURE    Requires shell 4.71 and up?
                          The page will be created when the property sheet is created, instead of the first time it is selected.
         PSP_RTLREADING   Windows 95 only: When this page is active, displays the text of pszTitle using right-to-left reading order on Hebrew or Arabic systems.
         PSP_USECALLBACK  Calls the function specified by pfnCallback when creating or destroying the property sheet page defined by this structure.
         PSP_USEHICON     Uses hIcon as the small icon on the tab for the page.
         PSP_USEICONID    Uses pszIcon as the name of the icon resource to load and use as the small icon on the tab for the page.
         PSP_USEREFPARENT Maintains the reference count specified by pcRefParent for the lifetime of the property sheet page created from this structure.
         PSP_USETITLE     Uses pszTitle as the title of the property sheet dialog box instead of the title stored in the dialog box template.
         *)
         Page.TPSP.dwFlags := PSP_DEFAULT or PSP_USECALLBACK or PSP_USEREFPARENT or PSP_USETITLE or PSP_DLGINDIRECT or PSP_USEHICON;
         if Page.Template.HasHelp then Page.TPSP.dwFlags := Page.TPSP.dwFlags or PSP_HASHELP;

         //Handle to the instance from which to load the dialog box template, icon, or title string resource.
         Page.TPSP.hInstance := {System.}hInstance; //Unused?

         //pszTemplate and pResource occupy the same space in the TPropSheetPage
         //DO NOT specify both!
//       if (Page.TPSP.dwFlags and PSP_DLGINDIRECT) = 0 then begin
            //Dialog box template to use to create the page.
            //This member can specify either the resource identifier of the template or
            //the address of a string that specifies the name of the template.
            //Meaningful only if dwFlags does not include the PSP_DLGINDIRECT value.
//          Page.TPSP.pszTemplate := MakeIntResource(psDialog); //psDialog is from DUTShellPropertySheet.inc
//       end else begin
            //Pointer to a dialog box template in memory.
            //Meaningful only if dwFlags includes the PSP_DLGINDIRECT value.
            Page.TPSP.pResource := Page.Template.DlgTemplate;
//       end; { else }

         //hIcon and pszIcon occupy the same space in the TPropSheetPage
         //DO NOT specify both!
//       if (Page.TPSP.dwFlags and PSP_USEHICON) = PSP_USEHICON then begin
            //Handle to the icon to use as the small icon in the tab for the page.
            //Meaningful only if dwFlags includes the PSP_USEHICON value.
            Page.TPSP.hIcon := Page.Icon.Handle; //Nil if Empty
//       end else if (Page.TPSP.dwFlags and PSP_USEICONID) = PSP_USEICONID then begin
            //Icon resource to use as the small icon in the tab for the page.
            //This member can specify either the identifier of the icon resource or
            //the address of a string that specifies the name of the icon resource.
            //Meaningful only if dwFlags includes the PSP_USEICONID value.
//          Page.TPSP.pszIcon := PChar;
//       end; { else }

         //Title of the property sheet dialog box.
         //This title overrides the title specified in the dialog box template.
         //This member can specify either the identifier of a string resource or
         //the address of a string that specifies the title.
         //Meaningful only if dwFlags includes the PSP_USETITLE value.
         Page.TPSP.pszTitle := PChar(Page.Title);

         //Pointer to the dialog box procedure for the page.
         Page.TPSP.pfnDlgProc := @PropSheetDlgProc;

         //Application-defined data.
         { Here we pass a pointer to the TPropertySheetPage instance }
         Page.TPSP.lParam := Integer(Page);

         //Extra application-defined data could be appended to the
         //TPropSheetPage record but this is dangerous since
         //Microsoft has already "expanded" the TPropSheetPage and TPropSheetHeader
         //structures when going to Internet Explorer 4 and then 5

         //Pointer to an application-defined callback function that is called
         //when the page is created and when it is about to be destroyed.
         //Meaningful only if dwFlags includes the PSP_USECALLBACK value.
         Page.TPSP.pfnCallBack := @PropSheetCallback;

         //Pointer to the reference count value.
         //Meaningful only if dwFlags includes the PSP_USERREFPARENT value.
         {
         Unfortunately, the COMObject's RefCount property is read-only and
         cannot be passed in.  So we implement our own DLLRefCount flag
         (which is updated by the shell thanks to this pointer) and
         check it in our DllCanUnloadNow function.
         }
         Page.TPSP.pcRefParent := @DLLRefCount;

         {
         Create the property sheet and retrieve its handle.
         }
         Page.HPSP := CreatePropertySheetPage(Page.TPSP);
         if Assigned(Page.HPSP) and not lpfnAddPage(Page.HPSP, lParam) then begin
            { Failed to add the page }
            DestroyPropertySheetPage(Page.HPSP);
            raise Exception.Create('IShellPropSheetExt.AddPages: Failed to create/add a page');
         end else begin
            Page.Template.Active := True;
            Inc(PageNumber);
         end; { then }
      until False;
      Result := S_OK;
   finally
      Page.Free;
   end; { try }
{$IFDEF Debug}
Loggit('AddPages ends');
{$ENDIF}
end; { IShellPropSheetExt.AddPages }

function TCOMObjShellPropSheetExt.ReplacePage(uPageID : UINT; lpfnReplaceWith : TFNAddPropSheetPage; lParam : LPARAM) : HResult; stdcall;
{
   Replaces a page in a property sheet for a control panel object.

uPageID
   Identifier of the page to replace.
   The values for this parameter for control panels can be found in the CPLEXT.H header file.

lpfnReplacePage
   Pointer to a function that the property sheet handler must call
   to replace a page of the property sheet.
   The function expects a property sheet handle (returned by a call
   to the CreatePropertySheetPage function) and
   the lParam parameter passed to the ReplacePage method.

lParam
   Parameter to pass to the function specified by lpfnReplacePage.
   It has no meaning to the method.

   To replace a page, a property sheet handler fills a PROPSHEETPAGE structure,
   calls CreatePropertySheetPage, and then
   calls the function specified by lpfnReplacePage.
}
begin
   Result := E_NOTIMPL;
end; { IShellPropSheetExt.ReplacePage }

end. { unit COMObjShellPropSheetExt }
