unit DUTDelShellUtils;
(*
   Daniel U. Thibault
   <D.U.Thibault@Bigfoot.com>
   6 September 1998
   Updated 17 February 1999
    1998-1999 Daniel U. Thibault.  All Rights Reserved.

   This is freeware.  No legally binding promises.  If you use the 
   component or any part of its code, you must include an 
   acknowledgement in the finished code or executable (in the About 
   box and/or the Help file).

   Supporting functions and procedures for the DUTDelShellExtensions.
*)
interface

   function GetDelFileNumber(const Name : String) : Integer;
{
   Read the "magic number" contained in the specified DEL file.
}

   function SetDelFileNumber(const Name: String; Num : Integer) : Integer;
{
   Sets the "magic number" contained in file Name to Num.
   Returns zero if successfull, a negative error number otherwise.
}

   function IncDelFileNumber(const Name: String; Num : Integer) : Integer;
{
   Increases the "magic number" contained in file Name by Num.
   Returns zero if successfull, a negative error number otherwise.
}

implementation

uses
   SysUtils; { Format, Exception }

function GetDelFileNumber(const Name : String) : Integer;
{
   Read the "magic number" contained in the specified DEL file.
}
var
   TF : TextFile;
begin
   AssignFile(TF, Name);
   try
      Reset(TF);
      try
         ReadLn(TF, Result);
      finally
         CloseFile(TF);
      end; { try }
   except
      on E : EInOutError do begin
         Result := -E.ErrorCode;
      end; { on }
   end; { try }
end; { GetDelFileNumber }

function SetDelFileNumber(const Name: String; Num : Integer) : Integer;
{
   Sets the "magic number" contained in file Name to Num.
   Returns zero if successfull, a negative error number otherwise.
}
var
   TF : TextFile;
begin
   AssignFile(TF, Name);
   try
      Rewrite(TF);
      try
         WriteLn(TF, Num);
      finally
         CloseFile(TF);
         Result := 0;
      end; { try }
   except
      on E : EInOutError do begin
         Result := -E.ErrorCode;
      end; { on }
   end; { try }
end; { SetDelFileNumber }

function IncDelFileNumber(const Name: String; Num : Integer) : Integer;
{
   Increases the "magic number" contained in file Name by Num.
   Returns zero if successfull, a negative error number otherwise.
}
begin
   Result := GetDelFileNumber(Name);
   if Result < 0 then Exit;
   Result := SetDelFileNumber(Name, Result + Num);
end; { IncDelFileNumber }

end. { unit DUTDelShellUtils }
