unit DUTRegUnreg;
(*
   Daniel U. Thibault
   <D.U.Thibault@Bigfoot.com>
   16 September 1998
   Updated 17 June 1999
    1998-1999 Daniel U. Thibault.  All Rights Reserved.

   This is freeware.  No legally binding promises.  If you use the 
   component or any part of its code, you must include an 
   acknowledgement in the finished code or executable (in the About 
   box and/or the Help file).
*)
interface

uses
   Windows, { HKEY_CLASSES_ROOT }
   Classes,
   Controls,
   Forms,
   StdCtrls;

type
   TForm1 = class(TForm)
      chkDEL: TCheckBox;
      chkContextMenu: TCheckBox;
      chkCopyHook: TCheckBox;
      chkDragDrop: TCheckBox;
      chkDrop: TCheckBox;
      chkIcon: TCheckBox;
      chkPropertySheet: TCheckBox;
      btnRegisterAll: TButton;
      btnUnregisterAll: TButton;
      btnDone: TButton;
      procedure FormCreate(Sender: TObject);
      procedure chkContextMenuClick(Sender: TObject);
      procedure chkCopyHookClick(Sender: TObject);
      procedure chkDragDropClick(Sender: TObject);
      procedure chkDropClick(Sender: TObject);
      procedure chkIconClick(Sender: TObject);
      procedure chkPropertySheetClick(Sender: TObject);
      procedure btnRegisterAllClick(Sender: TObject);
      procedure btnUnregisterAllClick(Sender: TObject);
      procedure btnDoneClick(Sender: TObject);
   private
   { Private declarations }
      procedure RegisterButtons;
   public
   { Public declarations }
   end; { TForm1 }

var
   Form1: TForm1;

implementation

{$R *.DFM}

uses
   Dialogs, { MessageDlg }
//   Registry,
   DUTUtilsReg,
   DUTUtilsFiles; { DUTGetSystemDirectory }

const
   DLLStr = 'DUTDelShellExtensions.dll';
   DUTShellContextMenu     = '{6A5EF9E1-DFCE-11CF-9A34-444553540000}';
   DUTShellCopyHook        = '{6A5EF9E2-DFCE-11CF-9A34-444553540000}';
   DUTShellPropertySheet   = '{6A5EF9E3-DFCE-11CF-9A34-444553540000}';
   DUTShellIconHandler     = '{6A5EF9E4-DFCE-11CF-9A34-444553540000}';
   DUTShellDragDropHandler = '{6A5EF9E5-DFCE-11CF-9A34-444553540000}';
   DUTShellDropHandler     = '{6A5EF9E6-DFCE-11CF-9A34-444553540000}';
   DelStr           = '\.del';
   DelShelStr       = '\DelShellFile';
   DelIconStr       = DelShelStr + '\DefaultIcon';
   DelShellexStr    = DelShelStr + '\shellex\';
   ContextMenuStr   = DelShellexStr + 'ContextMenuHandlers';
   PropertySheetStr = DelShellexStr + 'PropertySheetHandlers';
   DropStr          = DelShellexStr + 'DropHandler';
   IconHandlerStr   = DelShellexStr + 'IconHandler';
   DragDropStr      = '\Directory\shellex\DragDropHandlers';
   CopyHookStr1     = '\Directory\shellex\CopyHookHandlers';
   CopyHookStr2     = '\*\shellex\CopyHookHandlers';
   DUTContextMenuStr = '\DUTShellContextMenu';
   DUTCopyHookStr    = '\DUTShellCopyHook';
   DUTDragDropStr    = '\DUTShellDragDropHandler';
   DUTPropSheetStr   = '\DUTShellPropertySheet';

//var
//   TR : TRegistry;

procedure RegExt(const GUIDStr, Name, FileName : String);
begin
   SetINIString(HKEY_CLASSES_ROOT, '\CLSID\' + GUIDStr, '', Name);
   SetINIString(HKEY_CLASSES_ROOT, '\CLSID\' + GUIDStr + '\InprocServer32', '',               FileName);
   SetINIString(HKEY_CLASSES_ROOT, '\CLSID\' + GUIDStr + '\InprocServer32', 'ThreadingModel', 'Apartment');
end; { RegExt }

procedure TForm1.RegisterButtons;
begin
   btnRegisterAll.Enabled := not (chkContextMenu.Checked and
                                 chkCopyHook.Checked and
                                 chkDragDrop.Checked and
                                 chkDrop.Checked and
                                 chkIcon.Checked and
                                 chkPropertySheet.Checked);
   btnUnregisterAll.Enabled := (chkContextMenu.Checked or
                                 chkCopyHook.Checked or
                                 chkDragDrop.Checked or
                                 chkDrop.Checked or
                                 chkIcon.Checked or
                                 chkPropertySheet.Checked);
   if not btnUnregisterAll.Enabled then begin
      // Unregister DEL file type
//    TheRegistry.RootKey := HKEY_CLASSES_ROOT;
      TheRegistry.DeleteKey(DelStr);
      TheRegistry.DeleteKey(DelShelStr);
      chkDEL.State := cbUnchecked;
   end else begin
      // Register DEL file type
      SetINIString(HKEY_CLASSES_ROOT, DelStr,     '', Copy(DelShelStr, 2, Length(DelShelStr) - 1));
      SetINIString(HKEY_CLASSES_ROOT, DelShelStr, '', 'Magic Number File');
      chkDEL.State := cbChecked;
   end; { else }
end; { RegisterButtons }

procedure TForm1.FormCreate(Sender: TObject);
begin
   TheRegistry.RootKey := HKEY_CLASSES_ROOT;
   // DEL file type registered?
   if not TheRegistry.KeyExists(DelStr) then Exit;

   Enabled := False;
   chkDEL.Checked := True;
   // Context Menu handler
   chkContextMenu.Checked := TheRegistry.KeyExists(ContextMenuStr + DUTContextMenuStr);
   // Copy Hook handler
   chkCopyHook.Checked := TheRegistry.KeyExists(CopyHookStr1 + DUTCopyHookStr);
   // Drag-Drop handler
   chkDragDrop.Checked := TheRegistry.KeyExists(DragDropStr + DUTDragDropStr);
   // Drop handler
   chkDrop.Checked := TheRegistry.KeyExists(DropStr);
   // Icon handler
   chkIcon.Checked := TheRegistry.KeyExists(IconHandlerStr);
   // Property Sheet handler
   chkPropertySheet.Checked := TheRegistry.KeyExists(PropertySheetStr + DUTPropSheetStr);
   Enabled := True;

   RegisterButtons;
end;

procedure TForm1.chkContextMenuClick(Sender: TObject);
begin
   if chkContextMenu.Checked then begin
      // Context Menu handler
      SetINIString(HKEY_CLASSES_ROOT, ContextMenuStr + DUTContextMenuStr, '', DUTShellContextMenu);
      RegExt(DUTShellContextMenu, Copy(DUTContextMenuStr, 2, Length(DUTContextMenuStr) - 1), DUTGetSystemDirectory + DLLStr);
   end else begin
//    TheRegistry.RootKey := HKEY_CLASSES_ROOT;
      TheRegistry.DeleteKey(ContextMenuStr + DUTContextMenuStr);
      TheRegistry.DeleteKey('\CLSID\' + DUTShellContextMenu);
   end; { else }
   RegisterButtons;
end;

procedure TForm1.chkCopyHookClick(Sender: TObject);
begin
   if chkCopyHook.Checked then begin
      // Copy Hook handler
      SetINIString(HKEY_CLASSES_ROOT, CopyHookStr1 + DUTCopyHookStr, '', DUTShellCopyHook);
      SetINIString(HKEY_CLASSES_ROOT, CopyHookStr2 + DUTCopyHookStr, '', DUTShellCopyHook);
      RegExt(DUTShellCopyHook, Copy(DUTCopyHookStr, 2, Length(DUTCopyHookStr) - 1), DUTGetSystemDirectory + DLLStr);
   end else begin
//    TheRegistry.RootKey := HKEY_CLASSES_ROOT;
      TheRegistry.DeleteKey(CopyHookStr1 + DUTCopyHookStr);
      TheRegistry.DeleteKey(CopyHookStr2 + DUTCopyHookStr);
      TheRegistry.DeleteKey('\CLSID\' + DUTShellCopyHook);
      if Enabled then MessageDlg('You may need to restart Explorer for this to take effect.', mtCustom, [mbOK], 0);
   end; { else }
   RegisterButtons;
end;

procedure TForm1.chkDragDropClick(Sender: TObject);
begin
   if chkDragDrop.Checked then begin
      // Drag-and-Drop handler
      SetINIString(HKEY_CLASSES_ROOT, DragDropStr + DUTDragDropStr, '', DUTShellDragDropHandler);
      RegExt(DUTShellDragDropHandler, Copy(DUTDragDropStr, 2, Length(DUTDragDropStr)- 1), DUTGetSystemDirectory + DLLStr);
   end else begin
//    TheRegistry.RootKey := HKEY_CLASSES_ROOT;
      TheRegistry.DeleteKey(DragDropStr + DUTDragDropStr);
      TheRegistry.DeleteKey('\CLSID\' + DUTShellDragDropHandler);
   end; { else }
   RegisterButtons;
end;

procedure TForm1.chkDropClick(Sender: TObject);
begin
   if chkDrop.Checked then begin
      // Drop handler
      SetINIString(HKEY_CLASSES_ROOT, DropStr, '', DUTShellDropHandler);
      RegExt(DUTShellDropHandler, 'DUTShellDropHandler', DUTGetSystemDirectory + DLLStr);
      if Enabled then MessageDlg('You may need to restart Explorer for this to take effect.', mtCustom, [mbOK], 0);
   end else begin
//    TheRegistry.RootKey := HKEY_CLASSES_ROOT;
      TheRegistry.DeleteKey(DropStr);
      TheRegistry.DeleteKey('\CLSID\' + DUTShellDropHandler);
   end; { else }
   RegisterButtons;
end;

procedure TForm1.chkIconClick(Sender: TObject);
begin
   if chkIcon.Checked then begin
      // Icon handler
      SetINIString(HKEY_CLASSES_ROOT, IconHandlerStr, '', DUTShellIconHandler);
      SetINIString(HKEY_CLASSES_ROOT, DelIconStr,     '', '%1');
      RegExt(DUTShellIconHandler, 'DUTShellIconHandler', DUTGetSystemDirectory + DLLStr);
   end else begin
//    TheRegistry.RootKey := HKEY_CLASSES_ROOT;
      TheRegistry.DeleteKey(IconHandlerStr);
      TheRegistry.DeleteKey(DelIconStr);
      TheRegistry.DeleteKey('\CLSID\' + DUTShellIconHandler);
   end; { else }
   RegisterButtons;
end;

procedure TForm1.chkPropertySheetClick(Sender: TObject);
begin
   if chkPropertySheet.Checked then begin
      // Property Sheet handler
      SetINIString(HKEY_CLASSES_ROOT, PropertySheetStr + DUTPropSheetStr, '', DUTShellPropertySheet);
      RegExt(DUTShellPropertySheet, Copy(DUTPropSheetStr, 2, Length(DUTPropSheetStr) - 1), DUTGetSystemDirectory + DLLStr);
   end else begin
//    TheRegistry.RootKey := HKEY_CLASSES_ROOT;
      TheRegistry.DeleteKey(PropertySheetStr + DUTPropSheetStr);
      TheRegistry.DeleteKey('\CLSID\' + DUTShellPropertySheet);
   end; { else }
   RegisterButtons;
end;

procedure TForm1.btnDoneClick(Sender: TObject);
begin
   Application.Terminate;
end;

procedure TForm1.btnRegisterAllClick(Sender: TObject);
begin
   Enabled := False;
   chkContextMenu.Checked := True;
   chkCopyHook.Checked := True;
   chkDragDrop.Checked := True;
   chkDrop.Checked := True;
   chkIcon.Checked := True;
   chkPropertySheet.Checked := True;
   Enabled := True;
end;

procedure TForm1.btnUnregisterAllClick(Sender: TObject);
begin
   Enabled := False;
   chkContextMenu.Checked := False;
   chkCopyHook.Checked := False;
   chkDragDrop.Checked := False;
   chkDrop.Checked := False;
   chkIcon.Checked := False;
   chkPropertySheet.Checked := False;
   Enabled := True;
end;

end. { unit DUTRegUnreg }
