unit DUTShellContextMenu;
{..$DEFINE Debug}
(*
   Daniel U. Thibault
   <D.U.Thibault@Bigfoot.com>
   2 February 1999
   Updated 17 February 1999
    1999 Daniel U. Thibault.  All Rights Reserved.

   This is freeware.  No legally binding promises.  If you use the 
   component or any part of its code, you must include an 
   acknowledgement in the finished code or executable (in the About 
   box and/or the Help file).

   The shell ContextMenuHandler extension.
*)
interface

uses
   Windows,
   COMObj, { TComObjectFactory }
   COMObjContextMenu; { TCOMObjContextMenu }

const
   CLSID_DelContextMenu : TGUID = (
      D1:$6A5EF9E1; D2:$DFCE; D3:$11CF; D4:($9A, $34, $44, $45, $53, $54, $00, $00));

type
   TDUTShellContextMenu = class(TCOMObjContextMenu)
   protected
      function DoQuery : HResult; override;
(*
      function DoMenuMsg(uMsg : UINT; wParam : WPARAM; lParam : LPARAM) : HResult; override;
*)
   public
      //Event handler
      procedure ReportMagic(Sender : TObject);
   end; { TDUTShellContextMenu }

implementation

uses
{$IFDEF Debug}
DLLDebug, { Loggit }
Graphics, { TBitmap, TIcon }
Dialogs,  { MessageDlg }
{$ENDIF}
   SysUtils, { Format }
   COMServ,  { COMServer }
   DUTUtilsGraphics, { DuplicateBitmap }
   DUTDelShellUtils;

{ ########## TDUTShellContextMenu ########## }

function TDUTShellContextMenu.DoQuery : HResult;
{$IFDEF Debug}
var
   ASubMenu : TContextMenu;
   AnIcon   : Graphics.TIcon;
{$ENDIF}
begin
{$IFDEF Debug}
Loggit('TDUTShellContextMenu.DoQuery: Begins');
{$ENDIF}
   Result := inherited DoQuery; //Simply returns S_OK
   //Add one menu item to the context menu
   if FFiles.Count = 1 then begin
      FContextMenu.Add('ReportMagic',
                       '&Magic number...',
                       'Report the magic number of the selected file',
                       ReportMagic,
                       False {Checked}, False {Grayed}, True {Separator}, True {Visible});
   end else begin
{$IFNDEF Debug}
      FContextMenu.Add('ReportAvg',
                       '&Average magic number...',
                       'Report the average magic number of the selected files',
                       ReportMagic,
                       False {Checked}, False {Grayed}, True {Separator}, True {Visible});
{$ELSE}
      FContextMenu.Add('ReportAvg',
                       '&Alpha',
                       'Alpha Hint',
                       ReportMagic,
                       True {Checked}, False {Grayed}, False {Separator}, True {Visible});
      { Here's one way you could set the CheckedBitmap: }
//    FContextMenu.CheckedBitmaps[0] := LoadBitmap(0, PChar(OBM_DNARROW));
      { Here's another, showing how to use a TIcon }
      AnIcon := Graphics.TIcon.Create;
//    AnIcon.LoadFromFile('E:\ClipArt\Delphi\Images\Icons\Smiley.ico');
      AnIcon.LoadFromFile('C:\Clip Art\Delphi\Images\Icons\Stop.ico');
      FContextMenu.CheckedBitmaps[0] := DuplicateBitmap(TGraphic(AnIcon), True);
      AnIcon.Free;

      FContextMenu.Add('ReportAvg',
                       '&Beta',
                       'Beta Hint',
                       ReportMagic,
                       False {Checked}, False {Grayed}, False {Separator}, True {Visible});
      FContextMenu.Add('ReportAvg',
                       '&Gamma',
                       'Gamma Hint',
                       ReportMagic,
                       False {Checked}, False {Grayed}, False {Separator}, True {Visible});
      { We make Gamma an MF_BITMAP menu }
      FContextMenu.MenuBitmaps[2] := LoadBitmap(0, PChar(OBM_DNARROW));
      FContextMenu.Add('ReportAvg',
                       'Delta &Gamma',
                       'Delta-Gamma Hint',
                       ReportMagic,
                       False {Checked}, False {Grayed}, False {Separator}, True {Visible});
      { We make Delta-Gamma an MF_BITMAP menu too }
      FContextMenu.MenuBitmaps[3] := LoadBitmap(0, PChar(OBM_UPARROW));
      FContextMenu.Add('ReportAvg',
                       '&Epsilon',
                       'Epsilon Hint',
                       ReportMagic,
                       False {Checked}, False {Grayed}, False {Separator}, True {Visible});
      { We make Epsilon an MF_OWNERDRAW menu }
      FContextMenu.OwnerDrawnData[4] := 1;
      { Now we attach a submenu to Beta }
      ASubMenu := TContextMenu.Create;
      ASubMenu.Add('ReportAvg',
                       '&Sub-Beta',
                       'Sub-Beta Hint',
                       ReportMagic,
                       False {Checked}, False {Grayed}, False {Separator}, True {Visible});
      FContextMenu.Submenu[1] := ASubMenu;
{$ENDIF}
   end; { else }
// Result := S_OK;
{$IFDEF Debug}
Loggit('TDUTShellContextMenu.DoQuery: Ends');
{$ENDIF}
end; { TDUTShellContextMenu.DoQuery }

(*
function TDUTShellContextMenu.DoMenuMsg(uMsg : UINT; wParam : WPARAM; lParam : LPARAM) : HResult;
var
   Index : Integer;
begin
   //This simply sets Result to S_OK
   Result := inherited DoMenuMsg(uMsg, wParam, lParam);

   case uMsg of
   else
      MessageDlg('Not WM_MENUCHAR', mtCustom, [mbOK], 0);
   end; { case }
end; { TCOMObjContextMenu.DoMenuMsg }
*)

procedure TDUTShellContextMenu.ReportMagic(Sender : TObject);
{
   The event handler.  We put it in TDUTShellContextMenu for convenience.
}
var
   Index,
   NumBad,
   OneCount,
   TotalCount : Integer;
   Messg      : String;
begin
   //In this example, we display the "average magic number" of the DEL files.
   TotalCount := 0;
   NumBad := 0;
   for Index := 0 to (FFiles.Count - 1) do begin
      OneCount := GetDelFileNumber(FFiles[Index]);
      if OneCount < 0 then begin
         Inc(NumBad);
      end else begin
         Inc(TotalCount, OneCount);
      end; { else }
   end; { for }
   if FFiles.Count > 1 then begin
      if NumBad = 0 then begin
         Messg := Format('The %d selected files have an average magic number of %d', [FFiles.Count, TotalCount div FFiles.Count]);
      end else if NumBad = FFiles.Count then begin
         Messg := Format('The %d selected files all have a bad magic number', [NumBad]);
      end else begin
         Messg := Format('The %d selected files have an average magic number of %d (excluding %d with a bad magic number)', [FFiles.Count, TotalCount div (FFiles.Count - NumBad), NumBad]);
      end; { else }
   end else begin
      { FFiles.Count = 1 }
      if NumBad > 0 then begin
         Messg := Format('DelFile %s has a bad magic number', [FFiles[0]]);
      end else begin
         Messg := Format('DelFile %s has magic number %d', [FFiles[0], TotalCount]);
      end; { else }
   end;
   MessageBox(FhWnd, PChar(Messg), 'DUTShellContextMenu', MB_OK OR MB_ICONINFORMATION);
end; { TDUTShellContextMenu.ReportMagic }

initialization
   TComObjectFactory.Create(COMServer, TDUTShellContextMenu, CLSID_DelContextMenu, '', 'DUTShellContextMenu', ciMultiInstance);

end. { unit DUTShellContextMenu }
