unit DUTShellCopyHook;
{..$DEFINE Debug}
(*
   Daniel U. Thibault
   <D.U.Thibault@Bigfoot.com>
   30 January 1999
   Updated 18 August 1999
    1999 Daniel U. Thibault.  All Rights Reserved.

   This is freeware.  No legally binding promises.  If you use the 
   component or any part of its code, you must include an 
   acknowledgement in the finished code or executable (in the About 
   box and/or the Help file).

   The shell CopyHook extension.

   The CopyHook extension is different from the others in that
   it does not require multiple interfaces.
   It's also less useful than the other extensions, because it
   is so limited.
   CopyHook handlers can only prevent a folder or printer object
   from being copied, moved, deleted, or renamed.  The shell calls
   a CopyHook handler whenever file system directories are about
   to be copied, moved, deleted, or renamed, as well as whenever
   the status of a printer is about to change.
   This is in part because CopyHooks are registered only for the
   *, Directory and Printer file types.  But it seems to be also
   by design; that is, the Shell does not check the registry for
   copy hooks for files --not in Win95, anyway.
   A folder object can have multiple CopyHook handlers.

   The shell initializes the ICopyHook interface directly;
   that is, without using the IShellExtInit interface first.
   Explorer checks for copy hooks as needed, so you can add new
   ones dynamically, but once loaded a copy hook stays loaded until
   Explorer restarts --unregistering it isn't noticed.  This also
   means the DLL stays loaded until Explorer shuts down.

   But if the folder is moved etc. using a DOS box or something
   other than the shell, the CopyHook gets no notification...

   In this case,
   HKEY_CLASSES_ROOT\*\shellex\CopyHookHandlers\DUTShellCopyHook         = CLSID_DelCopy
   HKEY_CLASSES_ROOT\Directory\shellex\CopyHookHandlers\DUTShellCopyHook = CLSID_DelCopy
   HKEY_CLASSES_ROOT\Printers\shellex\CopyHookHandlers\DUTShellCopyHook  = CLSID_DelCopy

   Note that the copy hook doesn't really have anything to do with DEL
   files unless it is programmed to care --it'll be invoked for every
   damn shell operation.  In this example, we would have to run a search
   to find out if any DEL files were about to be affected, and only then
   prompt the user for confirmation...

   The Win32 Help states that:
   <<Clients register their id under STRREG_SHEX_COPYHOOK for file
   system hooks and STRREG_SHEX_PRNCOPYHOOK for printer hooks.>>
   But these constants don't mean anything to Delphi...
*)
interface

uses
   Windows,
   COMObj, { TComObjectFactory }
   COMObjCopyHook; { TCOMObjCopyHook }

const
   CLSID_DelCopy : TGUID = (
      D1:$6A5EF9E2; D2:$DFCE; D3:$11CF; D4:($9A, $34, $44, $45, $53, $54, $00, $00));
   //I'm guessing...
   REGSTR_SHEX_COPYHOOK =
      'Directory\shellex\CopyHookHandlers';
   STRREG_SHEX_COPYHOOK = REGSTR_SHEX_COPYHOOK;
   REGSTR_SHEX_PRNCOPYHOOK =
      'Printers\shellex\CopyHookHandlers';
   STRREG_SHEX_PRNCOPYHOOK = REGSTR_SHEX_PRNCOPYHOOK;

type
   TCopyHook = class(TCOMObjCopyHook)
   protected
      function DoCopyHook : UINT; override;
   end; { TCopyHook }

implementation

uses
{$IFDEF Debug}
DLLDebug, { Loggit }
{$ENDIF}
   COMServ; { COMServer }

{ ########## TCopyHook ########## }

function TCopyHook.DoCopyHook : UINT;
var
   Buffer : String;
begin
   case FOperation of
   chFileCopy   : Buffer := 'Request to copy ';
   chFileMove   : Buffer := 'Request to move ';
   chFileRename : Buffer := 'Request to rename ';
   chFileDelete : Buffer := 'Request to delete ';
   else
      Result := ID_YES;
      Exit;
   end; { case }
   Buffer := Buffer + FSource;
   if FOperation <> chFileDelete then begin
      Buffer := Buffer + ' to ' + FDest;
   end; { then }
   Buffer := Buffer + '.  Allow it?';
   Result := Windows.MessageBox(FhWnd, PChar(Buffer), 'Delphi Copy Hook Example', MB_ICONQUESTION or MB_YESNOCANCEL);
end; { DoCopyHook }

initialization
   TComObjectFactory.Create(COMServer, TCopyHook, CLSID_DelCopy, '', 'DUTShellCopyHook', ciMultiInstance);

end. { unit DUTShellCopyHook }
